/***
 * GASP: a gaming services platform for mobile multiplayer games.
 * MooDS: a communication protocol used in GASP
 *
 * Copyright (C) 2004,2005 CNAM/INT - 2006 CNAM/INT/Filao
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: gasp-team@objectweb.org
 *
 * Author: Romain Pellerin, Chanty Lim
 */
package org.mega.moods.generator;


/**
 * MooDS Generator v2.0
 * @author Romain PELLERIN
 */
public class LanguageTranslator {

    // languages:
    // 0: Java
    // 1: Flash
    public static final int JAVA = 0;
    public static final int FLASH = 1;
    
    // t: tableau de strings  crire
    // t[0]: packages
    // t[1]: imports
    // t[2]: class declaration
    // t[3]: attributes
    // t[4]: constructeur
    // t[5]: setters/getters
    // t[6]: end of class
    
    public static final String[] COMMENT = {"//","//"};

    
    public static String writeComments(int language, String comments){
    	String code = "";
    	if (language == JAVA) code = COMMENT[language]+comments;
    	return code;
    }
    
    
    public static String writePackage(int language, String packageName){
    	String code = "";
    	if (language == JAVA) code = "package "+ packageName + ";";
    	return code;
    }
    
    
    public static String writeImport(int language, String importName){
    	String code = "";
    	if (language == JAVA) code = "import "+ importName + ";";
    	return code;
    }
    
    
    public static String writeClassDeclaration(int language, String className, boolean isImplement, boolean isExtends, String _implements){
    	String code = "";
    	if (language == JAVA) {
    		code = "public class "+ className;
    		if (isImplement) code+= " implements " + _implements;
    		if (isExtends) code+= " extends " + _implements;
    		code+= " {";
    	}
    	
    	if (language == FLASH) code = "class "+ className + " {";
    	return code;
    }
    
    public static String writeConstructorDeclaration(int language, String constructorName, String[][] params){
    	return writeConstructorDeclaration(language, constructorName, params, false, null);
    }
    
    public static String writeConstructorDeclaration(int language,
            String constructorName, String[][] params,
            boolean isThrownException, String[] exceptions) {
        String code = "";
        if (language == JAVA) {
            code = "public " + constructorName + "(";
            if (params != null && params.length > 0)
                for (int i = 0; i < params.length; i++) {
                    code += getJavaType(params[i][0]) + " " + params[i][1];
                    if ((i + 1) != params.length)
                        code += ",";
                }
            code += ")";
            if (isThrownException) {
                code += " throws ";
                for (int i = 0; i < exceptions.length; i++) {
                    code += exceptions[i];
                    if (i + 1 < exceptions.length)
                        code += ", ";
                }
            }
            code += " {";
        }
        if (language == FLASH) {
            code = "public function " + constructorName + "(";
            if (params != null && params.length > 0)
                for (int i = 0; i < params.length; i++) {
                    code += params[i][1] + ":" + getFlashType(params[i][0]);
                    if ((i + 1) != params.length)
                        code += ",";
                }
            code += ") {";
        }
        return code;
    }
    
    public static String addGlobalVar(int language, 
    									boolean isPublic,
    									boolean isFinal, 
    									boolean isStatic,
    									boolean isToInit,
    									String initValue,
    									String type, 
    									String var){
    	String code = "";
    	if (language == JAVA){
    		if (isPublic) code += "public";
    		else code += "private";
    		if (isFinal) code += " final";
    		if (isStatic) code += " static";
    		code += " "+getType(JAVA, type);
    		code += " "+ var; 
    		if (isToInit){
	    		code += " = ";
    			if (initValue.equals(""))
    				code += getJavaAttributesInitiator(type);
    			else code += initValue;
    		}
    		code += ";";
    	}
    	
    	if (language == FLASH){
    		if (isPublic) code += "public";
    		else code += "private";
    		if (isStatic) code += " static";
    		code += " var "+ var;
    		code += ":"+getType(FLASH, type);
    		if (isToInit){
    			code += " = ";
    			if (initValue.equals(""))
    				code += "new " +getFlashAttributesInitiator(type);
    			else code += initValue;
    		}
    		code += ";";
    	}
    	return code;
    }
    	
    
    public static String writeFunctionDeclaration(int language,
    									boolean isPublic, 
    									boolean isStatic,
    									boolean isSynchronized,
    									boolean isVoid,
    									String returnType, 
    									String functionName, 
    									String[][] params, 
    									boolean isThrownException,
    									String[] exceptions){
    	String code = "";
    	if (language == JAVA){
    		if (isPublic) code += "public";
    		else  code += "private";
    		if (isStatic) code += " static";
    		if (isSynchronized) code += " synchronized";
    		if (isVoid) code += " void";
    		else code += " "+getType(JAVA, returnType);
    		code += " "+ functionName + "(";
    		
    		if (params!=null && params.length > 0){
	    		for (int i = 0; i<params.length; i++){
	    			code += getJavaType(params[i][0]) + " " + params[i][1];
	    			if ((i+1) != params.length) code += ",";
	    		}
    		}
    		
    		code += ")";
    		if (isThrownException){
    			code+=" throws ";
    			for (int i=0; i < exceptions.length ; i++){
    				code+= exceptions[i];
    				if (i+1<exceptions.length) code+=", ";
    			}
    		}
    		code += " {";
    	}
    	
    	if (language == FLASH){
    		if (isPublic) code += "public";
    		else code += "private";
    		if (isStatic) code += " static";
    		code += " function "+ functionName + "(";
    		
    		if (params!=null && params.length > 0){
	    		for (int i = 0; i<params.length; i++){
	    			code += params[i][1] + ":" + getFlashType(params[i][0]);
	    			if ((i+1) != params.length) code += ",";
	    		}
    		}
    		
    		code += ")";
    		if (!isVoid) code += ":"+getType(FLASH, returnType);
    		code += " {";
    	}
    	return code;
    }
    
    
   public static String stringLength(int language, String s){
    	String code = "";
    	if (language == JAVA){
    		code = s+".length()";
    	}
    	if (language == FLASH){
    		code = "length("+s+")";
    	}
    	return code;
    }

    
    public static String add(int language, 
    								boolean isToAdd,
    								String type,
    								String output, 
    								String[] itemsToAdd){
    	String code = "";
    	if (language == JAVA || language == FLASH){ 
    		code += output;
    		if (isToAdd) code += " += ";
    		else  code += " -= ";
    		if (itemsToAdd!=null) 
    			for (int i=0; i<itemsToAdd.length; i++){
	    			code += itemsToAdd[i];
	    			if ((i+1) != itemsToAdd.length) code += " + ";	
    			}
    		code += ";";
    	}
    	return code;
    }
    
    public static String addValueToValue(int language, 
			boolean isToAdd,
			String[] itemsToAdd){
		String code = "";
		if (language == JAVA || language == FLASH){ 
			code += "(";
			if (itemsToAdd!=null) 
				for (int i=0; i<itemsToAdd.length; i++){
					code += itemsToAdd[i];
					if ((i+1) != itemsToAdd.length) {
						if (isToAdd) code += " + ";
						else code += " - ";
					}
				}
			code += ")";
		}
		return code;
	}
    
    public static String increment(int language, boolean isToIncrement,String valueToIncrement){
		String code = "";
		if (language == JAVA || language == FLASH){ 
			if (isToIncrement) code += valueToIncrement+ "++;";
			else code += valueToIncrement+ "--;";
		}
		return code;
	}
    
    public static String addLocalVar(int language, boolean isToInit, String initValue, boolean isASpecificConstructor, String specificConstructor, String type, String var){
    	String code = "";
		if (language == JAVA){ 
			code += getType(JAVA,type) + " " + var;
			if (isToInit) {
				code += " = ";
				if (isASpecificConstructor) code += "new "+specificConstructor;
				else{
					if (initValue.equals("")) code += getJavaAttributesInitiator(type);
					else code += initValue;
				}
			}
			code += ";";
		}
		if (language == FLASH) {
			code += "var "+var+":"+getType(FLASH,type);
			if (isToInit) {
				code += " = ";
				
				if (isASpecificConstructor) code += "new "+specificConstructor;
				else{
					if (initValue.equals("")) code += " new " + getFlashAttributesInitiator(type);
					else code += initValue;
				}
			}
			
			code += ";";
		}
		return code;
    }
    
    public static String addLocalVar(int language, boolean isToInit, String initValue, String type, String var){
    	return addLocalVar(language,isToInit,initValue,false,"",type,var);
	}
    
    
    public static String testEqualType(int language, boolean isEqual, String type, String[] itemsToEquals){
    	String code = "";
    	if (language == JAVA){ 
    		String lType = getPrimitiveJavaType(type);
    		if (lType.equals("null") // for no primitive types
    				|| lType.equals("String")){
    			if (!isEqual) code += "!";
    			code += itemsToEquals[0];
    			code += ".equals(" + itemsToEquals[1] + ")";
    		}
    		else {
    			code += itemsToEquals[0];
    			if (isEqual) code += " == ";
    			else code += " != ";
    			code += itemsToEquals[1];

    		}
    		
    	}
    	if (language == FLASH){ 
    		if (type.equals("char")) code += replaceCharQuote(language,itemsToEquals[0]);
    		else code += itemsToEquals[0];
    		if (isEqual) code += "===";
    		else code += "!==";
    		if (type.equals("char")) code += replaceCharQuote(language,itemsToEquals[1]);
    		else code += itemsToEquals[1];
    	}
    	return code;
    }
    
    
    private static String replaceCharQuote(int language, String string) {
    	String stringParsed = "";
    	if (language == FLASH){
			//no ' for char but "
			
			for(int i = 0; i < string.length(); i++){
				if (string.charAt(i) == '\'')  stringParsed += '\"';
				else  stringParsed += string.charAt(i);
			}
		}
		return stringParsed;
	}


	public static String testSup(int language, boolean isSup, String[] itemsToTest){
    	String code = "";
    	if (language == JAVA ||language == FLASH){ 
    		code += itemsToTest[0];
    		if (isSup) code += ">";
    		else code += "<";
    		code += itemsToTest[1];
    	}
    	return code;
    }
    
    public static String testAnd(int language, boolean isToAnd, String[] itemsToTest){
    	String code = "";
    	if (language == JAVA ||language == FLASH){ 
    		code += itemsToTest[0];
    		if (isToAnd) code += " && ";
    		else code += " || ";
    		code += itemsToTest[1];
    	}
    	return code;
    }
    
    
    public static String ifDeclaration(int language, String _condition){
    	String code = "";
    	if (language == JAVA || language == FLASH){ 
    		code += "if (" +_condition+ ") {";;
    	}
    	return code;
    }
    
    
    public static String closeBlock(int language){
    	String code = "";
    	if (language == JAVA || language == FLASH){ 
    		code += "}" ;
    	}
    	return code;
    }
    
    
    public static String endOfLineMarker(int language){
    	String code = "";
    	if (language == JAVA || language == FLASH){ 
    		code += ";" ;
    	}
    	return code;
    }
    
    public static String elseDeclaration(int language, String _else){
    	String code = "";
    	if (language == JAVA || language == FLASH){ 
    		code += "else {\r" +_else;
    	}
    	return code;
    }
    
    public static String elseIfDeclaration(int language, String _condition){
    	String code = "";
    	if (language == JAVA || language == FLASH){ 
    		code += "else " + ifDeclaration(language, _condition);
    	}
    	return code;
    }
    
    
    public static String returnValue(int language, String valueToReturn){
    	String code = "";
    	if (language == JAVA || language == FLASH){ 
    		code += "return " + valueToReturn+";";
    	}
    	return code;
    }
    
    public static String arrayDeclaration(int language, String type, int dimension){
    	String code = "";
    	if (language == JAVA){ 
    		code += type;
    		for(int i = 0; i<dimension; i++){
    			code += "[]";
    		}
    	}
    	if (language == FLASH) code += "Array";
    	return code;	
    }
    
    public static String arrayLength(int language, String array){
    	String code = "";
    	if (language == JAVA || language == FLASH){ 
    		code += array+".length";
    	}
    	return code;	
    }
    
    
    public static String arrayAccess(int language, String tableName, String accessTo){
    	String code = "";
    	if (language == JAVA || language == FLASH){  
    		code+= tableName + "["+accessTo+"]";
    	}
        return code;
    }
    
    public static String arrayAccess(int language, String accessTo){
    	String code = "";
    	if (language == JAVA || language == FLASH){  
    		code+="["+accessTo+"]";
    	}
        return code;
    }
    
    public static String arrayInit(int language, String type, int dimension, String nb){
    	String code = "";
    	if (language == JAVA){ 
    		code += type;
    		code += "["+nb+"]";
    		for(int i = 0; i<dimension-1; i++){
    			 code += "[]";
    		}
    	}
    	if (language == FLASH){
    		code += "new Array()";
    	}
    	return code;	
    }
    
    public static String forLoopDeclaration(int language, String var, String min, String max){
    	String code = "";
    	if (language == JAVA){ 
    		code += "for(int "+ var +"="+ min +"; "+ var+" <"+ max +"; "+var+"++){";
    	}
    	if (language == FLASH){
    		code += "for(var "+ var +"="+ min +"; "+ var+" <"+ max +"; "+var+"++){";
    	}
    	return code;
    }
    
    public static String loopInterrupt(int language){
    	String code = "";
    	if (language == JAVA || language == FLASH) code += "break;";
    	return code;
    	
    }
    /*public static String forLoopEnd(int language){
    	String code = "";
    	if (language == JAVA || language == FLASH){ 
    		code += "}\r";
    	}
    	return code;
    }*/
    
    
    public static String testInstanceOfType(int language, String instance, String type){
    	String code = "";
    	if (language == JAVA || language == FLASH)
    		code += instance+" instanceof "+ type;
    	return code;
    }
    
    
    public static String affectValueToVar(int language, String var, String value){
    	String code = "";
    	if (language == JAVA || language == FLASH)
    		code += var + " = "+ value +";";
    	return code;
    }
    
    
    public static String callMethod(int language, 
    									boolean isAnObjectMethod,
    									String object,
    									String name,
    									String[] params){
    	String code = "";
    	if (language == JAVA || language == FLASH){
    		if (isAnObjectMethod) code += object + ".";
    		code += name + "(";
    		if (params != null)
	    		for (int i = 0; i<params.length; i++){
	    			code += params[i];
	    			if (i+1!=params.length) code += ",";
	    		}
    		code+=")";
    	}
    	return code;
    }
    
    
    public static String cast(int language, String type, String var){
    	String code = "";
    	if (language == JAVA)
    		code += "(("+getJavaObjectType(type)+") "+ var+")";
    	if (language == FLASH)
    		code += getFlashType(type)+"("+ var +")";
    	return code;
    }
    
    public static String castPrimitive(int language, String type, String var){
    	String code = "";
    	if (language == JAVA)
    		code += "("+getJavaType(type)+") "+ var;
    	if (language == FLASH)
    		code += getFlashType(type)+"("+ var +")";
    	return code;
    }
    
    
    public static String parseStringType(int language, String type, String stringToParse){
    	String code = "";
    	if (language == JAVA) 
    		code += getJavaTypeStringDecoder(getJavaType(type),stringToParse);
    	if (language == FLASH) 
    		code += getFlashTypeDecoder(getFlashType(type),stringToParse);
    	return code;
    }

    public static String whileLoopDeclaration(int language, String condition){
    	String code = "";
    	if (language == JAVA || language == FLASH ) 
    		code += "while ("+ condition + ") {";
    	return code;
    }
    
    public static String readBinaryType(int language, String type, String binaryVar){
    	String code = "";
    	if (language == JAVA) 
    		code += getJavaTypeDecoder(type,binaryVar);
    	return code;
    }
    
    
    public static String writeBinaryType(int language, String type, String valueToWrite, String binaryVar){
    	String code = "";
    	if (language == JAVA) 
    		code += getJavaTypeEncoder(type, valueToWrite, binaryVar);
    	return code;
    }
    
    
    // flash types String, Boolean, Number
    private static String getJavaTypeDecoder(String type, String binaryVar){
    	String code = "";
    	String javaType = getJavaType(type);
    	if (javaType.equals("long")) code += binaryVar+".readLong()";
    	if (javaType.equals("int")) code += binaryVar+".readInt()";
    	if (javaType.equals("short")) code += binaryVar+".readShort()";
    	if (javaType.equals("double")) code += binaryVar+".readDouble()";
    	if (javaType.equals("byte")) code += binaryVar+".readByte()";
    	if (javaType.equals("boolean")) code += binaryVar+".readBoolean()";
    	if (javaType.equals("String")) code += binaryVar+".readUTF()";
    	if (javaType.equals("char")) code += binaryVar+".readChar()";
    	return code;
    }
    
    
    
    private static String getJavaTypeEncoder(String type, String valueToWrite, String binaryVar){
    	String code = "";
    	String javaType = getJavaType(type);
    	if (javaType.equals("long")) code += binaryVar+".writeLong("+valueToWrite+");";
    	if (javaType.equals("int")) code +=  binaryVar+".writeInt("+valueToWrite+");";
    	if (javaType.equals("short")) code += binaryVar+".writeShort("+valueToWrite+");";
    	if (javaType.equals("double")) code += binaryVar+".writeDouble("+valueToWrite+");";
    	if (javaType.equals("byte")) code += binaryVar+".writeByte("+valueToWrite+");";
    	if (javaType.equals("boolean")) code += binaryVar+".writeBoolean("+valueToWrite+");";
    	if (javaType.equals("String")) code += binaryVar+".writeUTF("+valueToWrite+");";
    	if (javaType.equals("char")) code += binaryVar+".writeChar("+valueToWrite+");";
    	return code;
    }
    
    
    public static String getObjectTypeValue(int language, String type, String string) {
    	if (language == JAVA)
	    	return getJavaObjectTypeValue(type,string);
    	
    	return "";
	}

	private static String getJavaObjectTypeValue(String type, String s) {
    	String code = "";
    	String javaType = getJavaType(type);
    	if (javaType.equals("String")) code+= s;
    	if (javaType.equals("long")) code += "("+s+")"+".longValue()";
    	if (javaType.equals("int")) code += "("+s+")"+".intValue()";
    	if (javaType.equals("short")) code += "("+s+")"+".shortValue()";
    	if (javaType.equals("double")) code += "("+s+")"+".doubleValue()";
    	if (javaType.equals("byte")) code += "("+s+")"+".byteValue()";
    	if (javaType.equals("boolean")) code += "("+s+")"+".booleanValue()";
    	if (javaType.equals("char")) code += "("+s+")"+".charValue()";
    	return code;
	}



	// flash types String, Boolean, Number
    public static String getType(int language, String type){
    	if (language == JAVA)
	    	return getJavaType(type);
    	if (language == FLASH){
    		return getFlashType(type);
    	}
    
    	return "";
    }
    
    // flash types String, Boolean, Number
   
    public static String newType(int language, String type, String init){
    	return newType(language,type,new String[]{init});
    }
    
    public static String newType(int language, String type, String[] init){
    	String code = "";
    	if (language == JAVA || language == FLASH){
    		code += "new ";
    		code += getObjectType(language,type);
  
    		code += "(";
    		for(int i=0; i<init.length; i++){
    			code += init[i];
    			if (i != init.length - 1) code += ",";
    		}
    		code += ")";
    	}
    	return code;
    }
    

   
    
    private static String getObjectType(int language, String type) {
		if (language == JAVA) return getJavaObjectType(type);
		if (language == FLASH) return getFlashType(type);
		return null;
	}


	/**
     * Return the type of elementType.
     * 
     * @param elementType
     * @return id 
     */
    private static String getJavaObjectType(String elementType) {
       
    	if (elementType.toLowerCase().equals("xsd:string") ||
        		elementType.toLowerCase().equals("string")) 
            return "String";
        if (elementType.toLowerCase().equals("xsd:char")||
        		elementType.toLowerCase().equals("char")) 
            return "Char";
        if (elementType.toLowerCase().equals("xsd:int")||
        		elementType.toLowerCase().equals("int")) 
            return "Integer";
        if (elementType.toLowerCase().equals("xsd:long")||
        		elementType.toLowerCase().equals("long")) 
            return "Long";
        if (elementType.toLowerCase().equals("xsd:double")||
        		elementType.toLowerCase().equals("double")) 
            return "Double";
        if (elementType.toLowerCase().equals("xsd:short")||
        		elementType.toLowerCase().equals("short"))
            return "Short";
        if (elementType.toLowerCase().equals("xsd:byte")||
        		elementType.toLowerCase().equals("byte")) 
            return "Byte";
        if (elementType.toLowerCase().equals("xsd:boolean")||
        		elementType.toLowerCase().equals("boolean"))
            return "Boolean";
        else return elementType;
    } 
    

    // flash types String, Boolean, Number
    public static String getFlashType(String type){
    	String typeLowCase = type.toLowerCase();
    	if (typeLowCase.equals("xsd:string") 
    			|| typeLowCase.equals("string")
    		    || typeLowCase.equals("xsd:char")
    		    || typeLowCase.equals("char")) 
    		return "String";
    	else if (typeLowCase.equals("xsd:boolean")
            	|| typeLowCase.equals("boolean"))
        	return "Boolean";
    	else if (typeLowCase.equals("xsd:int")
        		|| typeLowCase.equals("int")
        	|| typeLowCase.equals("xsd:short")
        	    || typeLowCase.equals("short")
        	|| typeLowCase.equals("xsd:long")
        		|| typeLowCase.equals("long")
        	|| typeLowCase.equals("xsd:byte")
        		|| typeLowCase.equals("byte")
        	|| typeLowCase.equals("xsd:double")
        	|| typeLowCase.equals("double"))
        	return "Number";
        return type;
    }
    

    // java types String, boolean, long, int, short, double, byte
    public static String getJavaType(String type){
    	String lType = getPrimitiveJavaType(type);

    	if (lType.equals("null"))
    		return type;
    	else return lType;
    }
    
    
    // java types String, boolean, long, int, short, double, byte
    public static String getPrimitiveJavaType(String type){
    	String typeLowCase = type.toLowerCase();
    	if (typeLowCase.equals("xsd:string") 
    			|| typeLowCase.equals("string")) 
    		return "String";
    	if (typeLowCase.equals("xsd:char") 
    			|| typeLowCase.equals("char")) 
    		return "char";
    	else if (typeLowCase.equals("xsd:boolean")
            	|| typeLowCase.equals("boolean"))
        	return "boolean";
    	else if (typeLowCase.equals("xsd:int")
        		|| typeLowCase.equals("int"))
        	return "int";
    	else if (typeLowCase.equals("xsd:short")
        	    || typeLowCase.equals("short"))
        	return "short";
    	else if (typeLowCase.equals("xsd:long")
        		|| typeLowCase.equals("long"))
        	return "long";
        else if (typeLowCase.equals("xsd:byte")
        		|| typeLowCase.equals("byte"))
        	return "byte";
        else if (typeLowCase.equals("xsd:double")
        	|| typeLowCase.equals("double"))
        	return "double";
        else return "null";
    }
    
    // specific java methods
    private static String getJavaTypeStringDecoder(String type, String stringToParse){
    	String code = "";
    	String javaType = getJavaType(type);
    	if (javaType.equals("String")) code+= stringToParse;
    	if (javaType.equals("long")) code += "Long.parseLong("+stringToParse+")";
    	if (javaType.equals("int")) code += "Integer.parseInt("+stringToParse+")";
    	if (javaType.equals("short")) code += "Short.parseShort("+stringToParse+")";
    	if (javaType.equals("double")) code += "Double.parseDouble("+stringToParse+")";
    	if (javaType.equals("byte")) code += "Byte.parseByte("+stringToParse+")";
    	if (javaType.equals("boolean")) code += stringToParse;
    	if (javaType.equals("char")) code += "Char.parseChar("+stringToParse+")";
    	return code;
    }

    
    // specific flash methods
    private static String getFlashTypeDecoder(String type, String s){
    	if (type.equals("Number")) return "parseInt("+s+")";
    	if (type.equals("Boolean")) return s;
    	return s;
    }
    
    public static String getAttributesInitiator(int language, String type){
    	String code = "";
    	if (language == JAVA) code += getJavaAttributesInitiator(type);
    	if (language == FLASH) code += getFlashAttributesInitiator(type);
    	
    	return code;
    }
    
    // flash types String, Boolean, Number
    private static String getFlashAttributesInitiator(String typeFlash){
    	String typeLowCase = typeFlash.toLowerCase();
    	if (typeLowCase.equals("string")||typeLowCase.equals("char")) return "String(\"\")";
    	else if (typeLowCase.equals("boolean")) return "Boolean(false)";
    	else if (typeLowCase.equals("number")) return "Number(0)";
    	else if (containsTableMarker(FLASH,typeFlash)) return typeFlash;
    	return typeFlash+"()";
    }
    
    // flash types String, Boolean, Number
    private static String getJavaAttributesInitiator(String typeJava){
    	String typeLowCase = typeJava.toLowerCase();
    	if (typeLowCase.equals("string")) 
    		return "\"\"";
    	else if (typeLowCase.equals("char"))
        	return "\'\'";
    	else if (typeLowCase.equals("boolean"))
        	return "false";
    	else if (typeLowCase.equals("int")
        	    || typeLowCase.equals("short")
        		|| typeLowCase.equals("long")
        		|| typeLowCase.equals("byte")
        		|| typeLowCase.equals("double"))
        	return "0";
    	else if (containsTableMarker(JAVA,typeJava)) return "new " + typeJava;
    	else return "new "+typeJava+"()";
    }
    

    private static boolean containsTableMarker(int language, String type) {
    	if (language == JAVA){ 
	        if (type.indexOf('[') != -1) return true;
			else return false;
    	}
    	if (language == FLASH){
    		if (type.indexOf("Array") != -1) return true;
    		else return false;
    	}
    	return false;
	}


	public static String getLanguageFileExtension(int language) {
		if (language == JAVA) return "java";
		if (language == FLASH) return "as";
		return null;
	}
}
