/***
 * GASP: a gaming services platform for mobile multiplayer games.
 * MooDS: a communication protocol used in GASP
 *
 * Copyright (C) 2004,2005 CNAM/INT - 2006 CNAM/INT/Filao
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: gasp-team@objectweb.org
 *
 * Author: Romain Pellerin, Chanty Lim
 */
package org.mega.moods.generator;

import java.io.IOException;
import java.util.Hashtable;
import java.util.Vector;


/**
 * MooDS Generator v2.0
 * @author Chanthy LIM
 */
public class ServletGenerator {

    private static Vector classNames = new Vector();
    private static Hashtable stringsToWrite = new Hashtable();
    
    // languages:
    // 0: Java
    // 1: Flash
    public static final int JAVA = 0;
    public static final int FLASH = 1;
    
    public static final String SERVLET_SUFFIX = "Servlet";
    public static String servlet_complete_classname = "";
    public static String servlet_complete_path = "";

    // t: strings array to write
    // t[0]: packages
    // t[1]: imports
    // t[2]: class declaration
	// t[3]: comments
    // t[4]: init method
    // t[5]: doGet method
    // t[6]: doPost method
  
    public static void init(){
    	classNames = new Vector();
    	stringsToWrite = new Hashtable();
    }
    
    public static void startGenerateServlet(int language, String _package, String className) throws IOException{
    	String[] strings = new String[]{"","","","","","","", ""};
    	servlet_complete_path = _package + "/servlet/";
    	servlet_complete_classname = className + SERVLET_SUFFIX;
    	String linkClass = servlet_complete_path+servlet_complete_classname;
    	ClassMaker.createClassEntry(language,linkClass,servlet_complete_classname,strings,false);

	    classNames.add(linkClass);
    	
    	// strings to write table for that class
    	
    	
    	// init strings table

    	strings[0] = "/** \r"
			+" * MooDS Game Server Logic for "+((language == JAVA)? "Java":"")+".\r"
	 		+" * @author Generated by MooDS Generator v"+Generator.MOODS_VERSION
	 		+" - "+Generator.MOODS_YEAR+" "+Generator.MOODS_TEAM+" \r"
	 		+" */\r\r";
    	strings[0] += LanguageTranslator.writeComments(language,"This class has generated for the Web server.")+"\r";
    	strings[0] += LanguageTranslator.writePackage(language,_package+".servlet")+"\r";
        
    	// required imports
    	strings[1] += LanguageTranslator.writeImport(language,"java.io.DataInputStream")+"\r";
    	strings[1] += LanguageTranslator.writeImport(language,"java.io.DataOutputStream")+"\r"; 
    	strings[1] += LanguageTranslator.writeImport(language,"java.io.IOException")+"\r"; 
    	strings[1] += LanguageTranslator.writeImport(language,"java.util.Hashtable")+"\r\r"; 
    	strings[1] += LanguageTranslator.writeImport(language,"javax.servlet.ServletContext")+"\r";
    	strings[1] += LanguageTranslator.writeImport(language,"javax.servlet.ServletException")+"\r"; 
    	strings[1] += LanguageTranslator.writeImport(language,"javax.servlet.http.HttpServlet")+"\r";
    	strings[1] += LanguageTranslator.writeImport(language,"javax.servlet.http.HttpServletRequest")+"\r";
    	strings[1] += LanguageTranslator.writeImport(language,"javax.servlet.http.HttpServletResponse")+"\r";
    	strings[1] += LanguageTranslator.writeImport(language,_package+".server.*")+"\r"; 
    	strings[1] += LanguageTranslator.writeImport(language,_package+".sql.*")+"\r";

    	// class declaration
    	strings[2] += LanguageTranslator.writeClassDeclaration(language,servlet_complete_classname,false,true,"HttpServlet");
    	
    	// global attributes
    	strings[3] += "\tprivate static final String SELECT = \"select\";\r";
    	strings[3] += "\tprivate static final String UPDATE = \"update\";\r";
    	strings[3] += "\tprivate static final String INSERT = \"insert\";\r";
    	strings[3] += "\tprivate static final String DELETE = \"delete\";\r";
    	strings[3] += "\tCustomTypes ct = null;\r";
    	strings[3] += "\tSqlStorageManager sqlManager = null;\r";
    	
    	// constructor
    	strings[4] += "\t"+LanguageTranslator.writeFunctionDeclaration(language,	
				true,
				false,
				true,
				true,
				"",
				"init",
				new String[][]{},
				false,
				null)+"\r";
    	strings[4] += "\t\tServletContext app = getServletContext();\r";
    	strings[4] += "\t\tct = new CustomTypes();\r";
    	strings[4] += "\t\ttry {\r";
    	strings[4] += "\t\t\tboolean useDataSource = app.getInitParameter(\"moods.use.datasource\")!=null ? Boolean.valueOf(app.getInitParameter(\"moods.use.datasource\")).booleanValue() : false;\r";
    	strings[4] += "\t\t\tString webProjectName = app.getInitParameter(\"moods.web.home\") != null ? app.getInitParameter(\"moods.web.home\") : \"project\";\r";
    	strings[4] += "\t\t\tsqlManager = new SqlStorageManager(useDataSource, webProjectName);\r";
    	strings[4] += "\t\t} catch (Exception e) {\r";
    	strings[4] += "\t\t\te.printStackTrace();\r";
    	strings[4] += "\t\t}\r";
    	
     	// doGet method
    	strings[5] += "\t"+LanguageTranslator.writeFunctionDeclaration(language,	
    																	true,
    																	false,
    																	true,
    																	true,
    																	"",
    																	"doGet",
    																	new String[][]{	{"HttpServletRequest","request"},
				 																		{"HttpServletResponse","response"}
																					},
																		true,
																		new String[]{"IOException", "ServletException"})+"\r";
    	strings[5] += "\t\tdoPost(request, response);\r";

     	// doPost method
    	strings[6] += "\t"+LanguageTranslator.writeFunctionDeclaration(language,	
    																	true,
    																	false,
    																	true,
    																	true,
    																	"",
    																	"doPost",
    																	new String[][]{	{"HttpServletRequest","request"},
				 																		{"HttpServletResponse","response"}
																					},
																		true,
																		new String[]{"IOException", "ServletException"})+"\r";

    	strings[7] += LanguageTranslator.closeBlock(language);
    	stringsToWrite.put(linkClass,strings);
    	System.out.println("ServerGenerator: start to generate server at "+ servlet_complete_path+"Servlet");
    }
    
    
    public static void generateDoPostMethod(int language, String customType, TypesJAXBReader reader) throws Exception {	
    	generateDoPostBody(language, customType, reader);
    }
    
    public static void generateDoPostBody(int language, String customType, TypesJAXBReader reader) throws Exception {	
    	boolean selectPresent = reader.isRequestPresent(TypesJAXBReader.SELECT_TYPE);
    	boolean updatePresent = reader.isRequestPresent(TypesJAXBReader.UPDATE_TYPE);
    	boolean insertPresent = reader.isRequestPresent(TypesJAXBReader.INSERT_TYPE);
    	boolean deletePresent = reader.isRequestPresent(TypesJAXBReader.DELETE_TYPE);
    	if (selectPresent || updatePresent || insertPresent || deletePresent) {    		
	    	String[] strings = (String[]) stringsToWrite.get(servlet_complete_path+servlet_complete_classname);
	    	strings[6] += "\t\tDataInputStream dis = new DataInputStream(request.getInputStream());\r";
	    	strings[6] += "\t\tDataOutputStream dos = null;\r";
	    	strings[6] += "\t\ttry {\r";
	    	strings[6] += "\t\t\tdos = new DataOutputStream(response.getOutputStream());\r";
	    	strings[6] += "\t\t\tString requestType = request.getParameter(\"type\");\r";
	
	    	strings[6] += "\t\t\tif (SELECT.equals(requestType)) {\r";
	    	if (selectPresent) {
		    	strings[6] += "\t\t\t\tct.encodeData(sqlManager.select(), dos);\r";
	    	}
	    	strings[6] += "\t\t\t}\r";
	    	
	    	strings[6] += "\t\t\tif (UPDATE.equals(requestType)) {\r";
	    	if (updatePresent) {
		    	strings[6] += "\t\t\t\tsqlManager.update(ct.decodeData(dis));\r";
	    	}
	    	strings[6] += "\t\t\t}\r";
	    	
	    	strings[6] += "\t\t\tif (INSERT.equals(requestType)) {\r";
	    	if (insertPresent) {
		    	strings[6] += "\t\t\t\tsqlManager.insert(ct.decodeData(dis));\r";
	    	}
	    	strings[6] += "\t\t\t}\r";
	    	
	    	strings[6] += "\t\t\tif (DELETE.equals(requestType)) {\r";
	    	if (deletePresent) {
		    	strings[6] += "\t\t\t\tsqlManager.delete(ct.decodeData(dis));\r";
	    	}
	    	strings[6] += "\t\t\t}\r";
	    	
		    strings[6] += "\t\t} catch (Exception e) {\r";
	    	strings[6] += "\t\t\te.printStackTrace();\r";
			strings[6] += "\t\t}\r";
			strings[6] += "\t\tfinally{\r";
			strings[6] += "\t\t\tif (dis!=null) try{dis.close();}catch (Exception e){}\r";
			strings[6] += "\t\t\tif (dos!=null) try{dos.close();}catch (Exception e){}\r";
			strings[6] += "\t\t}\r";
	    	stringsToWrite.put(servlet_complete_path+servlet_complete_classname,strings);
    	}
    }
    
    // t: tableau de strings  crire
    // t[0]: packages
    // t[1]: imports
    // t[2]: class declaration
	// t[3]: public attributes
    // t[4]: private attributes
    // t[5]: constructor
    // t[6]: write required implemented methods: init, start, end, onStartEvent, onJoinEvent, onEndEvent, onQuitEvent, onDataEvent
    // t[7]: treatMessage method (specific because handles custom types objects)
    // t[8]: traditional utils methods
    // t[9]: end of class
    public static void finalize(int language) throws IOException{
   	 	System.out.println("ServerGenerator: finalize server file...");
    	String[] strings = (String[]) stringsToWrite.get(servlet_complete_path+servlet_complete_classname);
    	
    	if (!strings[0].equals("")) strings[0] += "\r";
    	if (!strings[1].equals("")) strings[1] += "\r";
    	if (!strings[2].equals("")) strings[2] += "\r\r";
    	if (!strings[3].equals("")) strings[3] += "\r";
    	if (!strings[4].equals("")) strings[4] = "\t// init method\r"+strings[4]+"\t"+LanguageTranslator.closeBlock(language)+"\r\r";
    	
    	if (!strings[5].equals(""))
    		strings[5] = "\t// doGet method\r"+strings[5]+"\t"+LanguageTranslator.closeBlock(language)+"\r\r";

    	if (!strings[6].equals(""))
    		strings[6] = "\t// doPost method\r"+strings[6]+"\t"+LanguageTranslator.closeBlock(language)+"\r\r";

    	if (!strings[7].equals(""))
    		strings[7] = strings[7]+"\r";
    	
   	 	System.out.println("ServletGenerator: server file finalized.");
    }
}