/***
 * GASP: a gaming services platform for mobile multiplayer games.
 * MooDS: a communication protocol used in GASP
 *
 * Copyright (C) 2004,2005 CNAM/INT - 2006 CNAM/INT/Filao
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: gasp-team@objectweb.org
 *
 * Author: Romain Pellerin, Chanty Lim
 */
package org.mega.moods.generator;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.w3._2001.xmlschema.ComplexType;


/**
 * MooDS Generator v2.0
 * @author Romain PELLERIN
 */
public class TypesGenerator {

    private static Vector classFiles = new Vector();
    private static Vector classNames = new Vector();
    private static Hashtable stringsToWrite = new Hashtable();
    
    // languages:
    // 0: Java
    // 1: Flash
    public static final int JAVA = 0;
    public static final int FLASH = 1;
    
    // t: tableau de strings  crire
    // t[0]: packages
    // t[1]: imports
    // t[2]: class declaration
	// t[3]: public attributes
    // t[4]: private attributes
    // t[5]: constructeur
    // t[6]: setters/getters
    // t[7]: end of class
    
    public static void init(){
    	classFiles = new Vector();
    	classNames = new Vector();
    	stringsToWrite = new Hashtable();
    }
    
    public static void startGenerateClass(int language, int encoding, String _package, String name, int currentTypeCommunicationModel) throws IOException{
    	
    	attributesAddedForCompactMode.removeAllElements();
    	
    	if (Generator.isTypesClassesAlreadyGenerated) return;
    	
    	boolean isACommonClass = (language==JAVA? true:false);
    	String path = Generator.getPath(language,isACommonClass);
    	String linkClass = path+name;
    	
    	// strings to write table for that class
    	String[] strings = new String[8];
    	ClassMaker.createClassEntry(language,linkClass,name,strings,isACommonClass);

	    classNames.add(linkClass);
    	
    	strings[0] = LanguageTranslator.writePackage(language,_package)+"\r";//"package "+ _package+";\r"; // (for java)	
        strings[1] = ""; // no imports
        strings[2] = ""; // 
        strings[3] = ""; // 
        strings[4] = ""; // 
        strings[5] = ""; // 
        strings[6] = ""; // 
        strings[7] = ""; // 
        	
    	strings[0] += "/** \r"
    			+" * MooDS Class File for "+((language == JAVA)? "Java":"Flash")+".\r"
    	 		+" * @author Generated by MooDS Generator v"+Generator.MOODS_VERSION
    	 		+" - "+Generator.MOODS_YEAR+" "+Generator.MOODS_TEAM+" \r"
    	 		+" */\r";

    	strings[2] += LanguageTranslator.writeClassDeclaration(language,name,false,false,"");
    	
    	String[][] params = null;
    	if (currentTypeCommunicationModel == Generator.UNICAST) params = new String[][]{{"short","_recipient"}};
    	else if (currentTypeCommunicationModel == Generator.MULTICAST) params = new String[][]{{LanguageTranslator.arrayDeclaration(language,"short",1),"_recipients"}};
    	
    	strings[5] += "\t"+LanguageTranslator.writeConstructorDeclaration(language,name,params)+"\r";//"\tpublic function "+name+"() {\r";
    
    	if (currentTypeCommunicationModel == Generator.UNICAST) strings[5] += "\t\t"+LanguageTranslator.affectValueToVar(language,"recipient","_recipient")+"\r";
    	else if (currentTypeCommunicationModel == Generator.MULTICAST) strings[5] += "\t\t"+LanguageTranslator.affectValueToVar(language,"recipients","_recipients")+"\r";
    	
    	strings[7] = LanguageTranslator.closeBlock(language);
    	
    	stringsToWrite.put(linkClass,strings);
    	System.out.println("TypesGenerator: start to generate class named "+ path+name);
    }
    
    private static Vector attributesAddedForCompactMode = new Vector();
    
    // t[0]: packages
    // t[1]: imports
    // t[2]: class declaration
    // t[3]: attributes
    // t[4]: constructeur
    // t[5]: setters/getters
    // t[6]: end of class
    public static void addClassAttribute(int language, boolean isTable, int dimension, boolean isCompactMode, String className, String attributeName, String type) throws IOException {
    	boolean isACommonClass = (language==JAVA);
    	String path = Generator.getPath(language,isACommonClass);
    	String[] strings = (String[]) stringsToWrite.get(path+className);
    	System.out.println("path: "+path+className+" strings: "+strings);
    	String lowAttributeName = Generator.lowCaseFirstLetter(attributeName);
    	
    	// attribute already added
    	if (isCompactMode && attributesAddedForCompactMode.contains(attributeName)) return;
    	
	    String languageType = (isTable? LanguageTranslator.arrayDeclaration(language,LanguageTranslator.getType(language,type),dimension):LanguageTranslator.getType(language,type));
	        
	    // attribute
	    strings[4] += "\t"+LanguageTranslator.addGlobalVar(language,false,false,false,false,"",languageType,lowAttributeName)+"\r";//"\t"+javaType+" "+lowAttributeName+";\r";
	    	
	    // constructor
	    if (!isTable && !((attributeName.equals("recipient") || attributeName.equals("recipients")))) strings[5] += "\t\t"+LanguageTranslator.affectValueToVar(language,lowAttributeName,LanguageTranslator.getAttributesInitiator(language,languageType))+"\r"; //lowAttributeName+" = "+getJavaAttributesInitiator(javaType)+";\r";
	    	
	    // setter
	    strings[6] += "\t"+ LanguageTranslator.writeFunctionDeclaration(language,true,false,false,false,languageType,"get"+Generator.upCaseFirstLetter(attributeName),null,false,null)+"\r";//public "+javaType+" get"+Generator.upCaseFirstLetter(attributeName)+"(){\r";
	    strings[6] += "\t\t"+LanguageTranslator.returnValue(language,lowAttributeName)+"\r";// return "+lowAttributeName+";\r";
	    strings[6] += "\t"+LanguageTranslator.closeBlock(language)+"\r\r";// }\r\r";
	    	
	    // getter
	    String[][] params = {{languageType,"_local_var"}};
	    strings[6] += "\t"+ LanguageTranslator.writeFunctionDeclaration(language,true,false,false,true,languageType,"set"+Generator.upCaseFirstLetter(attributeName),params,false,null)+"\r";//public "+javaType+" get"+Generator.upCaseFirstLetter(attributeName)+"(){\r";
	    strings[6] += "\t\t"+LanguageTranslator.affectValueToVar(language,lowAttributeName,"_local_var")+"\r";// return "+lowAttributeName+";\r";
	    strings[6] += "\t"+LanguageTranslator.closeBlock(language)+"\r\r";// }\r\r";

    	if (isCompactMode) attributesAddedForCompactMode.add(attributeName);
    	stringsToWrite.put(path+className,strings);
    	System.out.println("TypesGenerator: attribute ("+ type +","+ attributeName +") for "+ className +" class added");
    }
    

	// t[0]: packages
    // t[1]: imports
    // t[2]: class declaration
    // t[3]: attributes
    // t[4]: constructeur
    // t[5]: setters/getters
    // t[6]: end of class
    public static void finalize(int language, int i) throws IOException{
    	System.out.println("TypesGenerator: finalize "+classNames.get(i)+"...");
    	String[] strings = (String[]) stringsToWrite.get(classNames.get(i));
    	
    	if (!strings[0].equals("")) strings[0] += "\r";
    	if (!strings[1].equals("")) strings[1] += "\r";
    	if (!strings[2].equals("")) strings[2] += "\r\r";
    	if (!strings[3].equals("")) strings[3] += "\r";
    	if (!strings[4].equals(""))
    		strings[4] = "\t// attributes\r"+strings[4];
    	if (!strings[5].equals(""))
    		strings[5] = "\t// constructor\r"+strings[5]+"\t"+LanguageTranslator.closeBlock(language)+"\r\r";	
    	if (!strings[6].equals(""))
    		strings[6] = "\t// setter/getter methods\r"+strings[6]+"\r";
    	if (!strings[7].equals("")) strings[7] = strings[7]+"\r";
    	
    	System.out.println("TypesGenerator: "+classNames.get(i)+" finalized");
    }
    
    public static void finalizeTypes(int language) throws IOException{
    	 System.out.println("TypesGenerator: finalize types class files...");
    	 for (int i=0; i<classNames.size(); i++) {
    		 finalize(language,i);
         }
    	 System.out.println("TypesGenerator: types class finalized.");
    }
}
